#include <SPI.h>	// for SPI
#include <ESP8266WiFi.h>	// for WIFI

// 接続ピン設定
#define SLAVESELECTPIN	(15)

// Wi-Fi設定
const char* ssid = "4CE67645AC49";	// "your-ssid";
const char* password = "ktyn9r5vke7aa";	// "your-password";
IPAddress ip( 192, 168, 1, 94 );
WiFiServer g_server(80);

// デバッグ設定
//#define DEBUG	// 有効にするとシリアル出力が有効になります。
#ifdef DEBUG
#define Serialbegin Serial.begin
#define Serialprint Serial.print
#define Serialprintln Serial.println
#else
#define Serialbegin
#define Serialprint
#define Serialprintln
#endif

void setup()
{
	// SPI通信の初期化処理
	pinMode( SLAVESELECTPIN, OUTPUT );
	SPI.begin();
	SPI.setBitOrder( MSBFIRST );	// most-significant bit first

	// シリアル通信の初期化処理
	Serialbegin(9600);
	delay(10);

	// WiFiネットワーク接続
	Serialprint("Connecting to ");
	Serialprintln( ssid );
	WiFi.begin(ssid, password);
	while (WiFi.status() != WL_CONNECTED)
	{
		delay(500);
		Serialprint(".");
	}
	Serialprintln("");
	Serialprintln("WiFi connected");

	// 固定IPアドレスの設定
	WiFi.config( ip, WiFi.gatewayIP(), WiFi.subnetMask() );

	// WEBサーバー開始
	g_server.begin();
	Serialprintln("Server started");

	// IPアドレスの表示
	Serialprintln(WiFi.localIP());
}

void loop()
{
	WiFiClient client = g_server.available();
	if( client )
	{
		Serialprintln("new client");

		String strHttpRequest;
		ReadHttpRequest( client, strHttpRequest );
		if( 0 != strHttpRequest.length() )
		{	// HTTPリクエストあり
			Serialprintln( strHttpRequest );
			PrintHtml(client);
		}

		// Webブラウザに対して、データを取得するための時間を与える。
		delay(1);

		// コネクションを閉じる。
		client.stop();
		Serialprintln("client disconnected");
	}
}

// HTTPリクエスト読み込み
void ReadHttpRequest( WiFiClient& client, String& rstrHttpRequest )
{
	int iCounter = 0;	// HTTPリクエストがまったくないときに処理を抜ける用カウンタ。
	while( client.connected()
	  && 100 > iCounter )
	{
		iCounter++;
		if( !client.available() )
		{	// データが利用できるようになるまで待つ。
			Serialprint(".");
			// Webブラウザに対して、データを送信するための時間を与える。
			delay(1);
			continue;
		}
		// HTTPリクエストの一括読み込み
		rstrHttpRequest = client.readString();
		break;
	}
	return;
}

// HTML出力
void PrintHtml( WiFiClient& client )
{
	String str = "";

	str += "HTTP/1.1 200 OK\n"
	       "Content-Type: text/html\n"
	       "\n";

	str += "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\">\n"
	       "<html lang=\"ja\">\n"
	       "<head>\n"
	       "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n"
	       "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n"
	       "<title></title>\n"
	       "</head>\n"
	       "<body>\n";

	str += "Ch0 : " + String(GetValue(0)) + "<br /><br />\n";
	str += "Ch1 : " + String(GetValue(1)) + "<br /><br />\n";

	str += "<input type=\"button\" value=\"reload\" onclick=\"location.reload()\">\n"
	       "<br /><br />\n"
	       "</body>\n"
	       "</html>";

	client.print( str );
}

// アナログ値の取得
int GetValue( int channel )
{
	if( 0 != channel
	 && 1 != channel )
	{
		return 0;
	}

	digitalWrite( SLAVESELECTPIN, LOW );
	byte byteMSB;
	if( 0 == channel )
	{
		byteMSB = SPI.transfer(0b01101000);	// Start, SGL, Ch0, MBSF
	}
	else if( 1 == channel )
	{
		byteMSB = SPI.transfer(0b01111000);	// Start, SGL, Ch1, MBSF
	}
	byte byteLSB = SPI.transfer(0b00000000);
	digitalWrite( SLAVESELECTPIN, HIGH );
	// 上位ビットと下位ビットを連結し、10ビット分取り出す
	unsigned long ulValue = (( byteMSB << 8 ) + byteLSB ) & 0b0000001111111111;
	
	return (int)ulValue;
}
